# -*- coding: utf-8 -*-
# Copyright (C) 2020 Adrien Delle Cave
# SPDX-License-Identifier: GPL-3.0-or-later
"""xtrargon_ng.classes.abstract.modules"""

import abc
import logging
import threading

from six import string_types

from dwho.classes.modules import DWhoModuleBase

from .db import XtrargonDBBase
from ..lock_db import BackupsLockDB

from ..common import STATUS_UNKNOWN


LOG           = logging.getLogger('xtrargon_ng.classes.abstract.modules')

DB_URI_PARAMS = (('autocommit', '1'),
                 ('charset', 'utf8'),
                 ('cursorclass', 'Dict'),
                 ('wait_timeout', '2147483'))


class XtrargonModuleBase(DWhoModuleBase):
    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def MODULE_NAME(self):
        return

    def __init__(self):
        DWhoModuleBase.__init__(self)

        self.notifiers = None

        self._errors   = []
        self._state    = {}
        self.reset_state()

    def add_errors(self, errors):
        if isinstance(errors, string_types):
            errors = [errors]

        if not isinstance(errors, list):
            return

        for error in errors:
            if error not in self._errors:
                self._errors.append(error)

    def has_errors(self):
        return len(self._errors) > 0

    def reset_state(self):
        self._errors = []
        self._state  = {'status':     STATUS_UNKNOWN,
                        'started_at': None,
                        'ended_at':   None,
                        'uuid':       None,
                        'duration':   -1,
                        'server_id':  self.server_id,
                        'errors':     []}

    def get_state_value(self, key):
        return self._state.get(key)

    def set_state_value(self, key, value):
        self._state[key] = value

    def has_notifications(self, key = None):
        if not key:
            key = self.MODULE_NAME

        if self.config.get(key) \
           and self.config[key].get('notifications') \
           and self.notifiers:
            return True

        return False


class XtrargonDBModuleBase(XtrargonDBBase, XtrargonModuleBase):
    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def MODULE_NAME(self):
        return

    def __init__(self):
        XtrargonDBBase.__init__(self)
        XtrargonModuleBase.__init__(self)

        self._db_lock  = None

        self.lock_db   = None
        self.db_locked = None

    # pylint: disable-msg=attribute-defined-outside-init
    def init_db(self, module = None, default_cnf_file = 'client.cnf', host = None, port = None):
        self.lock_db   = threading.Event()
        self.db_locked = threading.Event()

        XtrargonDBBase.init_db(self, module, default_cnf_file, host, port)

    def _init_lock_db(self, config):
        if self.dbconf and self.dburi:
            self._check_db_server(self.dburi)
            if self._db_lock:
                self._terminate_lock_db()
            self._db_lock = BackupsLockDB(self, config)
            self._db_lock.daemon = True

    def _start_lock_db(self):
        if self._db_lock:
            self._db_lock.start()

    def _terminate_lock_db(self):
        if self._db_lock:
            self._db_lock.terminate()
            self._db_lock = None

    def _clear_lock_db(self):
        if self.lock_db:
            self.lock_db.clear()
        if self.db_locked:
            self.db_locked.clear()
